package sample;

import java.util.List;
import java.util.Random;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.cjk.CJKAnalyzer;
import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.queryParser.QueryParser;
import org.hibernate.Session;
import org.hibernate.search.FullTextSession;
import org.hibernate.search.Search;

import util.HibernateUtil;

public class Main {

	private static final String[] NAMES = {
		"田中一郎", "山田五郎", "中井秀雄", "山本太郎", "高橋純一"
	};
	private static final String[] TAGS = {
		"英語", "フランス語", "イタリア語", "中国語", "韓国語",
		"Java", "Ruby", "Perl", "C/C++", "PHP",
		"宴会部長", "鍋奉行",
		"プログラマー", "ITコンサルタント", "アーキテクト", "エバンジェリスト"
	};
	private static final int TAGS_MAX = 6;
	private static Random rand = new Random();
	private static Analyzer analyzer = new CJKAnalyzer();
	
	public static void main(String[] args) throws ParseException {
		if( args[0].equals( "insert" ) )
			insertEmployees();
		else if( args[0].equals( "list" ) )
			listEmployees();
		else if( args[0].equals( "update" ) )
			updateEmployees();
		else if( args[0].equals( "query" ) )
			queryEmployees();
		HibernateUtil.getSessionFactory().close();
	}
	
	private static void insertEmployees(){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();
		for( int i = 0; i < NAMES.length; i++ ){
			Employee employee = createEmployeeRandomTags( i );
			session.save( employee );
		}
		session.getTransaction().commit();
	}
	
	private static Employee createEmployeeRandomTags( int num ){
		Employee emp = new Employee();
		final String name = NAMES[num%NAMES.length];
		final int tagsNum = rand.nextInt( TAGS_MAX ) + 1;
		StringBuffer tags = new StringBuffer();
		for( int i = 0; i < tagsNum; i++ ){
			final String tag = TAGS[rand.nextInt( TAGS.length )];
			if( tags.toString().indexOf( tag ) < 0 )
				tags.append( tag ).append( ' ' );
		}
		emp.setName( name );
		emp.setTags( tags.toString() );
		return emp;
	}
	
	private static void listEmployees(){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();
		List employees = session.createQuery("from Employee").list();
		for( int i = 0; i < employees.size(); i++ ){
			Employee employee = (Employee)employees.get( i );
			System.out.println( employee.toString() );
		}
		session.getTransaction().commit();
	}
	
	private static void updateEmployees(){
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();
		List employees = session.createQuery("from Employee").list();
		for( int i = 0; i < employees.size(); i++ ){
			Employee employee = (Employee)employees.get( i );
			updateEmployeeRandomTags( employee );
			session.update( employee );
		}
		session.getTransaction().commit();
	}
	
	private static void updateEmployeeRandomTags( Employee emp ){
		final int tagsNum = rand.nextInt( TAGS_MAX ) + 1;
		StringBuffer tags = new StringBuffer();
		for( int i = 0; i < tagsNum; i++ ){
			final String tag = TAGS[rand.nextInt( TAGS.length )];
			if( tags.toString().indexOf( tag ) < 0 )
				tags.append( tag ).append( ' ' );
		}
		emp.setTags( tags.toString() );
	}
	
	private static void queryEmployees() throws ParseException{
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		FullTextSession fts = Search.createFullTextSession( session );
		fts.beginTransaction();
		QueryParser qp = new QueryParser( "tags", analyzer );
		for( String queryTag : TAGS ){
			System.out.println( "===== [" + queryTag + "] を検索 =====" );
			org.apache.lucene.search.Query lq = qp.parse( queryTag );
			org.hibernate.Query hq = fts.createFullTextQuery( lq );
			List result = hq.list();
			if( result.size() == 0 ){
				System.out.println( "該当者はみつかりませんでした。" );
				continue;
			}
			for( int i = 0; i < result.size(); i++ ){
				Employee employee = (Employee)result.get( i );
				System.out.println( employee.toString() );
			}
		}
		fts.getTransaction().commit();
	}
}
